// **************************************************
// *  CHOCO: an open-source Constraint Programming  *
// *     System for Research and Education          *
// *                                                *
// *    contributors listed in choco.Entity.java    *
// *           Copyright (C) F. Laburthe, 1999-2006 *
// **************************************************
package choco.integer.search;

import choco.AbstractProblem;
import choco.integer.IntDomainVar;
import choco.mem.IStateInt;

/**
 * A heuristic selecting the {@link choco.integer.var.IntDomainVarImpl} with smallest ration (domainSize / degree)
 * (the degree of a variable is the number of constraints linked to it)
 */
public final class DomOverDeg extends AbstractIntVarSelector {
  /**
   * the IIntVarSelector can be asked to return an {@link choco.integer.var.IntDomainVarImpl}
   *
   * @return a non instantiated search variable
   */
  public IntDomainVar selectIntVar() {
    int minDomSize = IStateInt.MAXINT;
    int maxDeg = 0;
    IntDomainVar v0 = null;


    if (null != vars) {
      int n = vars.length;
      for (int i = 0; i < n; i++) {
        IntDomainVar v = vars[i];
        if (!v.isInstantiated()) {
          int domSize = v.getDomainSize();
          int deg = v.getNbConstraints();
          if (minDomSize == IStateInt.MAXINT || domSize * maxDeg < minDomSize * deg) {
            minDomSize = domSize;
            maxDeg = deg;
            v0 = v;
          }
        }
      }
    } else {
      int n = problem.getNbIntVars();
      for (int i = 0; i < n; i++) {
        IntDomainVar v = (IntDomainVar) problem.getIntVar(i);
        if (!v.isInstantiated()) {
          int domSize = v.getDomainSize();
          int deg = v.getNbConstraints();
          if (minDomSize == IStateInt.MAXINT || domSize * maxDeg < minDomSize * deg) {
            minDomSize = domSize;
            maxDeg = deg;
            v0 = v;
          }
        }
      }
    }
    return v0;
  }

  public DomOverDeg(AbstractProblem pb) {
    problem = pb;

  }

  public DomOverDeg(AbstractProblem pb, IntDomainVar[] vs) {
    problem = pb;
    vars = vs;
  }

}
