// **************************************************
// *  CHOCO: an open-source Constraint Programming  *
// *     System for Research and Education          *
// *                                                *
// *    contributors listed in choco.Entity.java    *
// *           Copyright (C) F. Laburthe, 1999-2006 *
// **************************************************
package choco.integer.search;

import choco.AbstractProblem;
import choco.integer.IntDomainVar;
import choco.mem.IStateInt;

public final class MinDomain extends AbstractIntVarSelector {
  /**
   * the IIntVarSelector can be asked to return an {@link choco.integer.var.IntDomainVarImpl}
   *
   * @return a non instantiated search variable
   */
  public IntDomainVar selectIntVar() {
    int minDomSize = IStateInt.MAXINT;
    IntDomainVar v0 = null;
    if (null != vars) {
      int n = vars.length;
      for (int i = 0; i < n; i++) {
        IntDomainVar v = vars[i];
        if (!v.isInstantiated()) {
          int domSize = v.getDomainSize();
          if (domSize < minDomSize) {
            minDomSize = domSize;
            v0 = v;
          }
        }
      }
    } else {
      int n = problem.getNbIntVars();
      for (int i = 0; i < n; i++) {
        IntDomainVar v = (IntDomainVar) problem.getIntVar(i);
        if (!v.isInstantiated()) {
          int domSize = v.getDomainSize();
          if (domSize < minDomSize) {
            minDomSize = domSize;
            v0 = v;
          }
        }
      }
    }
    return v0;
  }

  public MinDomain(AbstractProblem pb) {
    problem = pb;
  }

  public MinDomain(AbstractProblem pb, IntDomainVar[] vs) {
    problem = pb;
    vars = vs;
  }
}
