// **************************************************
// *  CHOCO: an open-source Constraint Programming  *
// *     System for Research and Education          *
// *                                                *
// *    contributors listed in choco.Entity.java    *
// *           Copyright (C) F. Laburthe, 1999-2006 *
// **************************************************
package choco.integer.search;

import choco.Problem;
import choco.integer.IntDomainVar;

/**
 * An heuristic to first instantiating most constrained variables.
 */
public class MostConstrained extends AbstractIntVarSelector {
  /**
   * Searches for a non instantiated variable with as much constraints
   * posted on it as possible.
   * @return a non instantiated search variable
   */
  public IntDomainVar selectIntVar() {
    int maxDeg = 0;
    IntDomainVar v0 = null;
    if (null != vars) {
      int n = vars.length;
      for (int i = 0; i < n; i++) {
        IntDomainVar v = vars[i];
        if (!v.isInstantiated()) {
          int nbConstraints = v.getNbConstraints();
          if (nbConstraints > maxDeg) {
            maxDeg = nbConstraints;
            v0 = v;
          }
        }
      }
    } else {
      int n = problem.getNbIntVars();
      for (int i = 0; i < n; i++) {
        IntDomainVar v = (IntDomainVar) problem.getIntVar(i);
        if (!v.isInstantiated()) {
          int nbConstraints = v.getNbConstraints();
          if (nbConstraints > maxDeg) {
            maxDeg = nbConstraints;
            v0 = v;
          }
        }
      }
    }
    return v0;
  }

  /**
   * Builds the heuristic for the given problem.
   * @param pb the solved problem
   */
  public MostConstrained(final Problem pb) {
    problem = pb;
  }

  /**
   * Builds the heuristic for the given problem.
   * @param pb the solved problem
   * @param vs a list of variables instead of all prolem integer variables
   */
  public MostConstrained(final Problem pb, final IntDomainVar[] vs) {
    problem = pb;
    vars = vs;
  }
}
