// **************************************************
// *  CHOCO: an open-source Constraint Programming  *
// *     System for Research and Education          *
// *                                                *
// *    contributors listed in choco.Entity.java    *
// *           Copyright (C) F. Laburthe, 1999-2006 *
// **************************************************
package choco.mem;

import java.util.logging.Logger;

public class StoredLong {

  /**
   * Reference to an object for logging trace statements related memory & backtrack (using the java.util.logging package)
   */

  private static Logger logger = Logger.getLogger("choco.mem");

  /**
   * The current {@link choco.mem.Environment}.
   */

  private Environment environment;


  /**
   * Current value of the search.
   */

  private long currentValue;


  /**
   * The last world the search was moidified in.
   */

  int worldStamp;


  /**
   * The current {@link choco.mem.StoredIntTrail}.
   */

  private final StoredLongTrail trail;


  /**
   * Constructs a stored search with an unknown initial value.
   * Note: this constructor should not be used directly: one should instead
   * use the Environment factory
   */

  public StoredLong(Environment env) {
    this(env, 0);
  }


  /**
   * Constructs a stored search with an initial value.
   * Note: this constructor should not be used directly: one should instead
   * use the Environment factory
   */

  public StoredLong(Environment env, long d) {
    environment = env;
    currentValue = d;
    worldStamp = env.currentWorld;
    trail = (StoredLongTrail) this.environment.getTrail(Environment.LONG_TRAIL);
  }


  /**
   * Returns the current value.
   */

  public long get() {
    return currentValue;
  }


  /**
   * Checks if a value is currently stored.
   */

  public boolean isKnown() {
    return (currentValue != Double.NaN);
  }


  /**
   * Modifies the value and stores if needed the former value on the
   * trailing stack.
   */

  public void set(long y) {
    if (y != currentValue) {
      if (this.worldStamp < environment.currentWorld) {
        trail.savePreviousState(this, currentValue, worldStamp);
        worldStamp = environment.currentWorld;
      }
      currentValue = y;
    }
  }

  /**
   * modifying a StoredInt by an increment
   *
   * @param delta
   */
  public void add(long delta) {
    set(get() + delta);
  }

  /**
   * Modifies the value without storing the former value on the trailing stack.
   *
   * @param y      the new value
   * @param wstamp the stamp of the world in which the update is performed
   */

  void _set(long y, int wstamp) {
    currentValue = y;
    worldStamp = wstamp;
  }

  /**
   * Retrieving the environment
   */
  public Environment getEnvironment() {
    return environment;
  }

}
