//     ___  ___         PaLM library for Choco system
//    /__ \/ __\        (c) 2001 - 2004 -- Narendra Jussien
//   ____\ |/_____
//  /_____\/_____ \     PalmExplanation based constraint programming tool
// |/   (_)(_)   \|
//       \ /            Version 0.1
//       \ /            January 2004
//       \ /
// ______\_/_______     Contibutors: Franois Laburthe, Hadrien Cambazard, Guillaume Rochart...

package choco.palm.integer.constraints;

import choco.ContradictionException;
import choco.integer.IntDomainVar;
import choco.palm.*;
import choco.palm.integer.*;

import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: grochart
 * Date: 26 ao?t 2003
 * Time: 10:05:27
 * To change this template use Options | File Templates.
 */
public class PalmEqualXC extends AbstractPalmUnIntConstraint {
  protected final int cste;

  public PalmEqualXC(IntDomainVar v0, int cste) {
    this.v0 = v0;
    this.cste = cste;
    this.hook = ((ExplainedProblem) this.getProblem()).makeConstraintPlugin(this);
  }

  public String toString() {
    return this.v0 + " == " + this.cste;
  }

  public void propagate() throws ContradictionException {
    Explanation expl = ((ExplainedProblem) this.getProblem()).makeExplanation();
    ((ExplainedConstraintPlugin) this.getPlugIn()).self_explain(expl);
    ((ExplainedIntVar) this.v0).updateInf(this.cste, this.cIdx0, (Explanation) expl.copy());
    ((ExplainedIntVar) this.v0).updateSup(this.cste, this.cIdx0, expl);
  }

  public void awakeOnInf() {
  }

  public void awakeOnSup() {
  }

  public void awakeOnRem() {
  }

  public void awakeOnRestoreVal(int idx, int val) throws ContradictionException {
    if (val != this.cste) {
      Explanation expl = ((ExplainedProblem) this.getProblem()).makeExplanation();
      ((ExplainedConstraintPlugin) this.getPlugIn()).self_explain(expl);
      ((ExplainedIntVar) this.v0).removeVal(val, this.cIdx0, expl);
    }
  }

  public Boolean isEntailed() {
    if (this.v0.isInstantiatedTo(this.cste))
      return Boolean.TRUE;
    else if (!this.v0.canBeInstantiatedTo(this.cste)) return Boolean.FALSE;
    return null;
  }

  public boolean isSatisfied() {
    return this.v0.getVal() == this.cste;
  }

  public Set whyIsTrue() {
    Explanation expl = ((ExplainedProblem) this.getProblem()).makeExplanation();
    ((ExplainedIntVar) this.v0).self_explain(ExplainedIntDomain.DOM, expl);
    return expl.toSet();
  }

  public Set whyIsFalse() {
    Explanation expl = ((ExplainedProblem) this.getProblem()).makeExplanation();
    ((ExplainedIntVar) this.v0).self_explain(ExplainedIntDomain.VAL, this.cste, expl);
    return expl.toSet();
  }
}
