// **************************************************
// *  CHOCO: an open-source Constraint Programming  *
// *     System for Research and Education          *
// *                                                *
// *    contributors listed in choco.Entity.java    *
// *           Copyright (C) F. Laburthe, 1999-2006 *
// **************************************************
package choco.prop;

import choco.*;

import java.util.logging.Logger;

/**
 * An abstract class for all implementations of propagation engines.
 */
public abstract class AbstractPropagationEngine extends AbstractEntity implements PropagationEngine {

  /**
   * Storing the cause of the last contradiction.
   */

  protected Entity contradictionCause;

  /**
   * Reference to an object for logging trace statements related to propagation events (using the java.util.logging package)
   */

  protected static Logger logger = Logger.getLogger("choco.prop");


  public AbstractPropagationEngine(AbstractProblem problem) {
    super(problem);
  }

  /**
   * Erase the cause of the last contradiction.
   */

  public void setNoContradictionCause() {
    contradictionCause = null;
  }


  /**
   * Store the cause of the last contradiction.
   */

  public void setContradictionCause(Entity cause) {
    contradictionCause = cause;
  }


  /**
   * Retrieving the cause of the last contradiction.
   */

  public Entity getContradictionCause() {
    return contradictionCause;
  }

  /**
   * Gets the next queue from which a var will be propagated.
   */

  public EventQueue getNextActiveEventQueue() {
    return null;
  }
}
