// **************************************************
// *  CHOCO: an open-source Constraint Programming  *
// *     System for Research and Education          *
// *                                                *
// *    contributors listed in choco.Entity.java    *
// *           Copyright (C) F. Laburthe, 1999-2006 *
// **************************************************

package choco.prop;

import choco.*;
import choco.integer.IntDomainVar;
import choco.real.RealVar;
import choco.set.SetVar;

/**
 * An interface for all implementations of propagation engines.
 */
public interface PropagationEngine extends Entity {

  /**
   * Raising a contradiction with no cause.
   */

  public void raiseContradiction() throws ContradictionException;


  /**
   * Raising a contradiction with a cause.
   */

  public void raiseContradiction(Entity cause) throws ContradictionException;

  /**
   * Recording that there was no known cause for the last contradiction.
   */

  public void setNoContradictionCause();

  /**
   * Recording the cause of the last contradiction.
   */

  public void setContradictionCause(Entity cause);

  /**
   * Retrieving the cause of the last contradiction.
   */

  public Entity getContradictionCause();


  /**
   * Returns the next active var queue to propagate some events.
   * If it returns null, the propagation is finished.
   */

  public EventQueue getNextActiveEventQueue();

  /**
   * Removes all pending events (used when interrupting a propagation because
   * a contradiction has been raised)
   */
  public void flushEvents();

  /**
   * checking that the propagation engine remains in a proper state
   */
  public boolean checkCleanState();

  void postUpdateInf(IntDomainVar v, int idx);

  void postUpdateSup(IntDomainVar v, int idx);

  void postInstInt(IntDomainVar v, int idx);

  void postRemoveVal(IntDomainVar v, int x, int idx);

  void postUpdateInf(RealVar v, int idx);

  void postUpdateSup(RealVar v, int idx);

  void postRemEnv(SetVar v, int idx);

  void postAddKer(SetVar v, int idx);

  void postInstSet(SetVar v, int idx);

  boolean postConstAwake(Propagator constraint, boolean init);

  void registerEvent(ConstraintEvent event);

  VarEventQueue getVarEventQueue();
}
