// **************************************************
// *  CHOCO: an open-source Constraint Programming  *
// *     System for Research and Education          *
// *                                                *
// *    contributors listed in choco.Entity.java    *
// *           Copyright (C) F. Laburthe, 1999-2006 *
// **************************************************
package choco.search;

import choco.AbstractProblem;
import choco.ContradictionException;
import choco.branch.AbstractBranching;
import choco.branch.IntBranching;

import java.util.logging.Level;

/**
 * An abstract class for all implementations of branching objets (objects controlling the tree search)
 */
public abstract class AbstractIntBranching extends AbstractBranching implements IntBranching {

  /**
   * a method performing the exploration at the choice point level
   *
   * @deprecated replaced by the management incremental search (with a stack of BranchingTrace storing the
   *             environment (local variables) associated to each choice point
   */
  public boolean explore(int n) {
    AbstractProblem pb = manager.problem;
    Object v = selectBranchingObject();
    if (v != null) {
      try {
        return branchOn(v, n);
      } catch (ContradictionException e) {
        return false;
      }
    } else if (nextBranching != null) {
      return ((IntBranching) nextBranching).explore(n);
    } else {
      manager.recordSolution();
      manager.showSolution();
      return manager.stopAtFirstSol;
    }
  }

  public void goDownBranch(Object x, int i) throws ContradictionException {
    logDownBranch(x, i);
  }

  public void goUpBranch(Object x, int i) throws ContradictionException {
    logUpBranch(x, i);
  }

  protected void logDownBranch(final Object x, final int i) {
    if (logger.isLoggable(Level.FINE)) {
      int n = manager.problem.getEnvironment().getWorldIndex();
      if (n <= manager.loggingMaxDepth) {
//        logger.fine(getLogPrefix(n) + " [" + n + "]" + "on " + x + " down branch " + i);
        logger.log(Level.FINE, LOG_DOWN_MSG, new Object[]{new Integer(n),x,getDecisionLogMsg(i), new Integer(i)});
      }
    }
  }
  
  protected void logUpBranch(final Object x, final int i) {
    if (logger.isLoggable(Level.FINE)) {
      int n = manager.problem.getEnvironment().getWorldIndex();
      if (n <= manager.loggingMaxDepth) {
//        logger.fine(getLogPrefix(n) + " [" + n + "]" + "on " + x + " up branch " + i);
        logger.log(Level.FINE, LOG_UP_MSG, new Object[]{new Integer(n + 1),x,getDecisionLogMsg(i), new Integer(i)});
      }
    }
  }
}
