package choco.test.integer;

import choco.Problem;
import choco.integer.IntDomainVar;
import choco.integer.IntVar;
import choco.integer.constraints.extension.LargeRelation;
import choco.integer.constraints.extension.TuplesTest;
import choco.integer.search.RandomIntValSelector;
import choco.integer.search.RandomIntVarSelector;
import junit.framework.TestCase;

import java.util.ArrayList;

// **************************************************
// *                   J-CHOCO                      *
// *   Copyright (C) F. Laburthe, 1999-2003         *
// **************************************************
// *  an open-source Constraint Programming Kernel  *
// *     for Research and Education                 *
// **************************************************

public class NaryRelationTest extends TestCase {

  public void test1() {
    Problem pb = new Problem();
    IntVar x = pb.makeEnumIntVar("x", 1, 5);
    IntVar y = pb.makeEnumIntVar("y", 1, 5);
    IntVar z = pb.makeEnumIntVar("z", 1, 5);
    pb.post(pb.relationTuple(new IntVar[]{x, y, z}, new NotAllEqual()));
    pb.solveAll();
    assertEquals(120, pb.getSolver().getNbSolutions());
  }

  public void test2() {
    Problem pb = new Problem();
    IntVar x = pb.makeEnumIntVar("x", 1, 5);
    IntVar y = pb.makeEnumIntVar("y", 1, 5);
    IntVar z = pb.makeEnumIntVar("z", 1, 5);
    pb.post(pb.relationTuple(new IntVar[]{x, y, z}, (LargeRelation) (new NotAllEqual()).getOpposite()));
    pb.solveAll();
    assertEquals(5, pb.getSolver().getNbSolutions());
  }

  private class NotAllEqual extends TuplesTest {

    public boolean checkTuple(int[] tuple) {
      for (int i = 1; i < tuple.length; i++) {
        if (tuple[i - 1] != tuple[i]) return true;
      }
      return false;
    }

  }

  public void test3() {
    Problem pb = new Problem();
    IntVar x = pb.makeEnumIntVar("x", 1, 5);
    IntVar y = pb.makeEnumIntVar("y", 1, 5);
    IntVar z = pb.makeEnumIntVar("z", 1, 5);
    ArrayList forbiddenTuples = new ArrayList();
    forbiddenTuples.add(new int[]{1, 1, 1});
    forbiddenTuples.add(new int[]{2, 2, 2});
    forbiddenTuples.add(new int[]{3, 3, 3});
    forbiddenTuples.add(new int[]{4, 4, 4});
    forbiddenTuples.add(new int[]{5, 5, 5});
    pb.post(pb.infeasTuple(new IntVar[]{x, y, z}, forbiddenTuples));
    pb.solveAll();
    assertEquals(120, pb.getSolver().getNbSolutions());
  }

  public void test3bis() {
    Problem pb = new Problem();
    IntVar x = pb.makeEnumIntVar("x", 1, 5);
    IntVar y = pb.makeEnumIntVar("y", 1, 5);
    IntVar z = pb.makeEnumIntVar("z", 1, 5);
    ArrayList forbiddenTuples = new ArrayList();
    forbiddenTuples.add(new int[]{1, 1, 1});
    forbiddenTuples.add(new int[]{2, 2, 2});
    forbiddenTuples.add(new int[]{2, 5, 3});
    pb.post(pb.infeasTuple(new IntVar[]{x, y, z}, forbiddenTuples));
    pb.solveAll();
    assertEquals(122, pb.getSolver().getNbSolutions());
  }

  public void test3bisbis() {
    Problem pb = new Problem();
    IntVar v1 = pb.makeEnumIntVar("v1", 0, 2);
    IntVar v2 = pb.makeEnumIntVar("v2", 0, 4);
    ArrayList feasTuple = new ArrayList();
    feasTuple.add(new int[]{1, 1}); // x*y = 1
    feasTuple.add(new int[]{2, 4}); // x*y = 1
    pb.post(pb.feasTuple(new IntVar[]{v1, v2}, feasTuple));
    pb.solve();
    do {
      System.out.println("v1 : " + ((IntDomainVar) v1).getVal() + " v2: " + ((IntDomainVar) v2).getVal());
    } while (pb.nextSolution() == Boolean.TRUE);
    assertEquals(2, pb.getSolver().getNbSolutions());
  }

  public void test4() {
    Problem pb = new Problem();
    IntVar x = pb.makeEnumIntVar("x", 1, 5);
    IntVar y = pb.makeEnumIntVar("y", 1, 5);
    IntVar z = pb.makeEnumIntVar("z", 1, 5);
    ArrayList forbiddenTuples = new ArrayList();
    int cpt = 0;
    for (int i = 1; i <= 5; i++) {
      for (int j = 1; j <= 5; j++) {
        for (int k = 1; k <= 5; k++) {
          if (i != j || i != k || k != j) {
            int[] tuple = new int[3];
            tuple[0] = i;
            tuple[1] = j;
            tuple[2] = k;
            cpt++;
            forbiddenTuples.add(tuple);
          }
        }
      }
    }
    pb.post(pb.infeasTuple(new IntVar[]{x, y, z}, forbiddenTuples));
    pb.solveAll();
    assertEquals(5, pb.getSolver().getNbSolutions());
  }

  public void test5() {
    Problem pb = new Problem();
    IntVar x = pb.makeEnumIntVar("x", 1, 5);
    IntVar y = pb.makeEnumIntVar("y", 1, 5);
    IntVar z = pb.makeEnumIntVar("z", 1, 5);
    ArrayList goodTuples = new ArrayList();
    goodTuples.add(new int[]{1, 1, 1});
    goodTuples.add(new int[]{2, 2, 2});
    goodTuples.add(new int[]{3, 3, 3});
    goodTuples.add(new int[]{4, 4, 4});
    goodTuples.add(new int[]{5, 5, 5});
    pb.post(pb.feasTuple(new IntVar[]{x, y, z}, goodTuples));
    pb.solveAll();
    assertEquals(5, pb.getSolver().getNbSolutions());
  }

  //
  // un petit problme pos en sixime  la petite soeur de Ludivine :
  // trouvez l'ge de mes trois enfants :
  //     - le produit de leurs ages est gal  36
  //     - la somme de leurs ages est gal  13
  //     - mon an est blond (info cruciale)
  // ici il est rsolu de manire compltement bidon (on a dj la solution avant
  // de lancer le solve en ayant limin tous les tuples infaisables :))
  public void test6() {
    Problem pb = new Problem();
    IntDomainVar x = pb.makeEnumIntVar("x", 1, 12);
    IntDomainVar y = pb.makeEnumIntVar("y", 1, 12);
    IntDomainVar z = pb.makeEnumIntVar("z", 1, 12);
    ArrayList forbiddenTuplesProduct = new ArrayList();
    ArrayList forbiddenTuplesAine = new ArrayList();
    ArrayList symetryTuples = new ArrayList();
    for (int i = 1; i <= 12; i++) {
      for (int j = 1; j <= 12; j++) {
        for (int k = 1; k <= 12; k++) {
          int[] tuple = new int[3];
          tuple[0] = i;
          tuple[1] = j;
          tuple[2] = k;
          if (i * j * k != 36)
            forbiddenTuplesProduct.add(tuple);
          if (i > j || j > k || i > k)
            symetryTuples.add(tuple);
          if ((i == j && i > k) || (i == k && j > k) || (j == k && k > i))
            forbiddenTuplesAine.add(tuple);
        }
      }
    }

    pb.post(pb.eq(pb.sum(new IntVar[]{x, y, z}), 13));
    pb.post(pb.infeasTuple(new IntVar[]{x, y, z}, forbiddenTuplesProduct));
    pb.post(pb.infeasTuple(new IntVar[]{x, y, z}, forbiddenTuplesAine));
    pb.post(pb.infeasTuple(new IntVar[]{x, y, z}, symetryTuples));
    pb.solveAll();
    System.out.println("x " + x.getVal() + " y " + y.getVal() + " z " + z.getVal());
    assertEquals(1, pb.getSolver().getNbSolutions());
    assertEquals(2, x.getVal());
    assertEquals(2, y.getVal());
    assertEquals(9, z.getVal());
  }

  public static void genereCst(ArrayList tuples, int n) {
    int[] tuple = new int[n];
    int k = 0;
    for (int i = 0; i < n; i++)
      tuple[i] = 1;
    tuple[0] = 0;
    while (k < n) {
      tuple[k]++;
      if (tuple[k] > n) {
        tuple[k] = 1;
        k++;
      } else {
        if (testDouble(tuple)) {
          int[] t = new int[n];
          System.arraycopy(tuple, 0, t, 0, tuple.length);
          tuples.add(t);
        }
        k = 0;
      }
    }
  }


  public static boolean testDouble(int[] tuple) {
    for (int i = 0; i < tuple.length; i++) {
      for (int j = i + 1; j < tuple.length; j++) {
        if (tuple[i] == tuple[j])
          return true;
      }
    }
    return false;
  }


  public void test7() {
    Problem pb = new Problem();
    int n = 7;
    IntVar[] reines = new IntVar[n];
    IntVar[] diag1 = new IntVar[n];
    IntVar[] diag2 = new IntVar[n];
    //Definition Variables
    for (int i = 0; i < n; i++) {
      reines[i] = pb.makeEnumIntVar("reine-" + i, 1, n);
    }
    for (int i = 0; i < n; i++) {
      diag1[i] = pb.makeEnumIntVar("diag1-" + i, -n, 2 * n);
      diag2[i] = pb.makeEnumIntVar("diag2-" + i, -n, 2 * n);
    }
    //Tests contraintes N-aires
    ArrayList tuples = new ArrayList();
    genereCst(tuples, n);
    pb.post(pb.infeasTuple(reines, tuples));
//       pb.post(pb.infeasTuple(reines, tuples, 2001)); TODO: que voulait dire le parametre 2001 ?

    //Definition Contraintes restantes
    for (int i = 0; i < n; i++) {
      pb.post(pb.eq(diag1[i], pb.plus(reines[i], i)));
      pb.post(pb.eq(diag2[i], pb.minus(reines[i], i)));

      for (int j = i + 1; j < n; j++) {
        // pb.post( pb.neq(reines[i],reines[j]));
        pb.post(pb.neq(diag1[i], diag1[j]));
        pb.post(pb.neq(diag2[i], diag2[j]));
      }
    }
    // Resolution
    pb.getSolver().setValSelector(new RandomIntValSelector(110));
    pb.getSolver().setVarSelector(new RandomIntVarSelector(pb, 110));
    pb.solveAll();
    assertEquals(40, pb.getSolver().getNbSolutions());
  }

}
