package choco.test.real;

import choco.Problem;
import choco.Solver;
import choco.real.RealExp;
import choco.real.RealVar;
import choco.real.exp.*;
import choco.real.search.*;
import junit.framework.TestCase;

public class TrigoTests extends TestCase {
  public void test1() {
    Problem pb = new Problem();
    RealVar alpha = pb.makeRealVar("alpha", -Math.PI, Math.PI);

    RealExp exp = new RealMinus(pb,
        new RealCos(pb, alpha),
        new RealSin(pb, alpha));
    pb.post(pb.makeEquation(exp, pb.cst(0.0)));

    boolean first = false;
    Solver solver = pb.getSolver();
    solver.setFirstSolution(first);
    solver.generateSearchSolver(pb);
    solver.addGoal(new AssignInterval(new CyclicRealVarSelector(pb), new RealIncreasingDomain()));
    solver.launch();

    assertTrue(solver.getNbSolutions() >= 2);
    assertTrue(Math.abs(Math.cos(alpha.getInf()) - Math.sin(alpha.getInf())) < 1e-8);
  }

  public void test2() {
    Problem pb = new Problem();

    RealVar alpha = pb.makeRealVar("alpha", -5.5 * Math.PI, -1.5 * Math.PI);

    RealExp exp = new RealCos(pb, alpha);
    pb.post(pb.makeEquation(exp, pb.cst(1.0)));

    boolean first = false;
    Solver solver = pb.getSolver();
    solver.setFirstSolution(first);
    solver.generateSearchSolver(pb);
    solver.addGoal(new AssignInterval(new CyclicRealVarSelector(pb), new RealIncreasingDomain()));
    solver.launch();

    assertTrue(solver.getNbSolutions() >= 2);
  }
}
